#include <stdio.h>
#include "opencv2/opencv.hpp"

#define PIXVAL(iplimagep, x, y) (*(uchar *)((iplimagep)->imageData + (y) * (iplimagep)->widthStep + (x)))

int
draw_histogram(IplImage *mono, IplImage *disp, int *histwork)
{
    int i, j;
    
    for (i = 0; i < 255; i++) {
        histwork[i] = 0;
    }
    for (j = 0; j < mono->height; j++) {
        for (i = 0; i < mono->width; i++) {
            histwork[PIXVAL(mono, i, j)]++;
        }
    }

    cvCvtColor(mono, disp, CV_GRAY2BGR);
    for (i = 0; i < 255; i++) {
        cvLine(disp,
               cvPoint(i, mono->height - 1),
               cvPoint(i, mono->height - histwork[i]/32),
               CV_RGB(0, 255, 0));
    }

    return 0;
}

int
main()
{
    int i, j;
    IplImage *frame, *mono, *mono2, *disp, *disp2;
    int hist[256];
    int p1 = 0, p2 = 255;
    
    CvCapture *capture = cvCaptureFromCAM(0);
    if (capture == NULL) {
        fprintf(stderr, "cannot capture\n");
        return 1;
    }

    frame = cvQueryFrame(capture);
    mono = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);  // mono
    mono2 = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);  // mono transformed
    disp = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 3); // display
    disp2 = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 3); // display transformed

    cvNamedWindow("display", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("display transformed", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("frame", CV_WINDOW_AUTOSIZE);

    cvCreateTrackbar("p1", "display transformed", &p1, 255, NULL);
    cvCreateTrackbar("p2", "display transformed", &p2, 255, NULL);

    while (1) {
        frame = cvQueryFrame(capture); /* cannot be modified */

        cvCvtColor(frame, mono, CV_BGR2GRAY);
        draw_histogram(mono, disp, hist);

        if (p2 <= p1) {
            p2 = p1 + 1;
        }
        for (j = 0; j < mono->height; j++) {
            for (i = 0; i < mono->width; i++) {
                int v = PIXVAL(mono, i, j);
                if (v < p1) {
                    PIXVAL(mono2, i, j) = 0;
                } else if (p1 <= v && v <= p2) {
                    PIXVAL(mono2, i, j) = (255 * (v - p1)) / (p2 - p1);
                } else {
                    PIXVAL(mono2, i, j) = 255;
                }
            }
        }
        draw_histogram(mono2, disp2, hist);
        cvRectangle(disp,
               cvPoint(p1, mono->height - 3),
               cvPoint(p2, mono->height - 1),
                    CV_RGB(255, 0, 0), CV_FILLED);


        cvShowImage("display", disp);
        cvShowImage("display transformed", disp2);
        cvShowImage("frame", frame);

        if (cvWaitKey(30) > 0) {
            break;
        }
    }
    cvSaveImage("saved.jpg", frame);

    return 0;
}
